/***************************************************************************
 *   Copyright (C) 2004 by Raphael Langerhorst                             *
 *   raphael-langerhorst@gmx.at                                            *
 *                                                                         *
 *   Permission is hereby granted, free of charge, to any person obtaining *
 *   a copy of this software and associated documentation files (the       *
 *   "Software"), to deal in the Software without restriction, including   *
 *   without limitation the rights to use, copy, modify, merge, publish,   *
 *   distribute, sublicense, and/or sell copies of the Software, and to    *
 *   permit persons to whom the Software is furnished to do so, subject to *
 *   the following conditions:                                             *
 *                                                                         *
 *   The above copyright notice and this permission notice shall be        *
 *   included in all copies or substantial portions of the Software.       *
 *                                                                         *
 *   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,       *
 *   EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF    *
 *   MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.*
 *   IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR     *
 *   OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, *
 *   ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR *
 *   OTHER DEALINGS IN THE SOFTWARE.                                       *
 ***************************************************************************/
 
#include "GAttractAgent.h"

#include <GObject.h>
#include <GElement.h>
#include <GVector3.h>
#include <GForm.h>
#include <GWorldData.h>

#include <qmutex.h>
#include <qdom.h>

#include <exception>

using namespace std;

using namespace GCS;

namespace GBE
{

GAttractAgent::GAttractAgent()
// : GCS::GAgent(NULL,"GBE::GAttractAgent")
{
}


GAttractAgent::~GAttractAgent()
{
}

void GAttractAgent::receiveInfluence(const GElementInfluence& influence)
{
  if (requestObject()->hasForm() && requestObject()->hasEnergy())
  {
    
    const GElement* sender = this->requestObject()->getWorldData()->read(influence.source());
    
    const GForm* sender_form = sender->getObject()->getForm();
    const GEnergy* sender_energy = sender->getObject()->getEnergy();
    
//     qDebug("influence has position, attracting to element");
    
    //determine match...
    const GEnergy* energy = requestEnergy();
    
    double diff_level = energy->level() - sender_energy->level();
    double sum_sigma = energy->sigma() + sender_energy->sigma();
    double diff_amount = energy->amount() - sender_energy->amount();
    
    if (diff_level<0)
      diff_level = -diff_level;
      
    if (diff_amount<0)
      diff_amount = -diff_amount;
    
    if (diff_level<sum_sigma && sum_sigma>0 && energy->amount()>0 && sender_energy->amount()>0)
    {
      const GForm* f = requestForm();
      
      GVector3 direction = sender_form->Position - f->Position;
      
      if (direction.length() > 0)
      {
        
        bool ok;
        double attractfactor = xmlGetDouble("/dynamics/attractionfactor",ok);
        if (!ok)
        {
          qWarning("Couldn't load attraction factor from element data, initializing to default: 1");
          initFactor(1);
        }
        
        //THIS formula defines attraction speed
        double diff_speed = attractfactor*(sum_sigma - diff_level)/sum_sigma * sender_energy->amount();
        
        direction.normalize();
        direction.mul(diff_speed);
        
//         qDebug("calculated speed difference is " + QString::number(diff_speed));

        //timing mechanism: there is no timing involved because
        // it probably only depends on the amount of influence brought in;        

//         QDomElement e = xmlGetElement(xmlGetTopElement("dynamics",ok),"translationspeed",ok);
        GVector3 TranslationSpeed = this->xmlGetVector3("/dynamics/translationspeed",ok);
        
        TranslationSpeed += direction;
        
        this->xmlSetVector3("/dynamics/translationspeed",TranslationSpeed,ok);
        
        emit this->formChanged(*f);
      }
    }
  }
}

void GAttractAgent::initFactor(double factor)
{
  bool ok;
  xmlSetDouble("/dynamics/attractionfactor",factor,ok);
  if (!ok)
    qWarning("Couldn't initialize attraction factor!");
}

};
